      program gaussfit
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! A program to superpose like-molecule assemblies of peptides/proteins !
! by the reference algorithm, which involves the enumeration of all    !
! mappings of the chains of the superposed on those of the reference   !
! assembly, and Algorihm 1/Algorithm 2 described in Liwo & Lesniewski, !
! Molecules, 2025, submitted.                                          !
!                                                                      !
! Usage:                                                               !
! prog pdbfile.pdb [sigma2]                                            !
!                                                                      !
! where prog is the program name,                                      !
!                                                                      !
! pdbfile.pdb is the PDB file containing all structures to superpose   !
!              of which all pairs will be considered, chains separated !
!              with TER and structures separated with ENDMDL           !
!              (mandatory, otherwise the program will crash). The file !
!              must have the .pdb extension.                           !
!                                                                      !
! sigma2 is sigma**2 in Equation (7) of the paper. Must be entered     !
!        if the program was compiled for Algorithm 2, ignored for      !
!        Algorithm 1.                                                  !
!                                                                      !
! See Makefile for the explanation of the compiler flags to compile    !
! the program for Algorithm 1 or Algorithm 2, respectively.            !
!                                                                      !
! Each variant of the program also runs the reference algorithm, which !
! involves the enumeration of all chain permutations; therefore, the   !
! number of chains is limited to 10. The reference algorithm can be    !
! deleted but this will require code modification.                     !
!                                                                      !
! The output will be placed in pdfile.out_{options}_fitsq              !
! _o - the ORIENT flag turned on to find initial orientation & mapping !
!      (not recommended)                                               !
! _q - the INIQUAT flag turned on to find the initial orientation      !
!      & mapping (recommended)                                         !
! _p - chain-chain distances sorted in search of the best mapping      !
!      (recommended).                                                  !
!                                                                      !
! For Algorithm 1 (no Gaussfit flag) the output table is like this     !
!                                                                      !
!    pairs    RMSD_1    RMSD_d    RMSD_r    N/A                        !
!    1    2   7.14739   7.42368   6.61022   0.00000                    !
!    1    3   5.74924   6.25324   5.74924   0.00000                    !
!    1    4   6.75357   7.14406   6.41320   0.00000                    !
!                                                                      !
! For Algorithm 2 (the GAUSSFIT flag) the output table is like this    !
!                                                                      !
!    pairs    RMSD_Phi  RMSD_d    RMSD_r    RMSD_1*                    !
!    1    2   6.55743   7.17818   6.61022   7.14739                    !
!    1    3   6.56711   5.79996   5.74924   5.74924                    !
!    1    4   6.77281   6.88364   6.41320   6.75357                    !
!                                                                      !
! RMSD_1* is usually equal to RMSD_1 (as happens in the excerpts from  !
! the two outputs shown above) but it is obtained by applying SVD with !
! the chain-to-chain mapping obtained after running Algorithm 2.       !
!                                                                      !
! v. 1.0 February 6, 2025
!                                                                      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      implicit none
      include "DIMENSIONS"
      include "sizesclu.dat"
      include "COMMON.CONTROL"
      include "COMMON.NAMES"
      include "COMMON.CHAIN"
      include "COMMON.CLUSTER"
      include "COMMON.IOUNITS"
      include 'COMMON.GAUSSWIDTH'
      double precision rmsd_dist,rmsd_gauss,rmsd_fitsq
      common /RMSD/ rmsd_dist,rmsd_fitsq
      integer i,j,k,icon,jcon,ii,ncon,iretcode,nfun,imin
      character*16 karta
      double precision rmsd
      double precision difconf,difconf_gauss
      real start,finish
      real rmsdtab(4,maxconf*(maxconf-1)/2)
      
      call initialize
      call getarg(1,pdbname)
      call getarg(2,karta)
#ifdef GAUSSFIT
      read (karta,*) sigma2
      print *,"sigma2",sigma2
#else
      sigma2=1.0d0
#endif
      prefix=pdbname(:index(pdbname,'.')-1)
      outname=prefix(:len_trim(prefix))//'.'//'out'
#ifdef ORIENT
      outname=outname(:len_trim(outname))//'_o'
#endif
#ifdef INIQUAT
      outname=outname(:len_trim(outname))//'_q'
#endif
#ifdef GAUSSFIT
      outname=outname(:len_trim(outname))//'_g'
#endif
#ifdef PAIR
      outname=outname(:len_trim(outname))//'_p'
#endif
      outname=outname(:len_trim(outname))//'_fitsq'
      open (iout,file=outname)
#ifdef GAUSSFIT
      write (iout,'(80(1h*)//1h*,5x,a73,1h*//1h*,5x,a73,1h*//80(1h*))')
     & "Superposition of peptide/protein oligomers with Algorithm 2",
     & "Liwo and Lesniewski, Molecules, 2025, submitted"
      write (iout,*) "sigma2=",sigma2
#else
      write (iout,'(80(1h*)//1h*,5x,a73,1h*//1h*,5x,a73,1h*//80(1h*))')
     & "Superposition of peptide/protein oligomers with Algorithm 1",
     & "Liwo and Lesniewski, Molecules, 2025, submitted"
#endif
      call cpu_time(start)
      print *,"Opening ",pdbname(:len_trim(pdbname))
      write (iout,*) "Opening ",pdbname(:len_trim(pdbname))
      open(ipdbin,file=pdbname,status="old")
      do icon=1,maxconf
        call read_pdb(icon,*10,*20)
      enddo
   20 ncon=icon-1
      close(ipdbin)
      write (iout,*) "ncon",ncon
c      print *,"Opening ",pdb2
c      open(ipdbin,file=pdb2,status="old")
c      call read_pdb(2,*10)
c      close(ipdbin)
#ifdef DEBUG
      print *,"nchain",nchain
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (iout,'(a3,3i4,3f10.5,5x,3f10.5)') restyp(itype(j)),j,
     &     j-chain_border(1,i)+1,i,allcart(:,j,icon),allcart(:,j,jcon)
        enddo
        write (iout,*) 
      enddo
#endif
      call cpu_time(finish)
      write (iout,*) "Time to read data",finish-start," s"
      start=finish
      ii=0
      do icon=1,ncon-1
        do jcon=icon+1,ncon
          ii = ii+1
          rmsd_gauss=difconf_gauss(icon,jcon)
          rmsdtab(1,ii)=rmsd_gauss
          rmsdtab(2,ii)=rmsd_dist
          rmsdtab(4,ii)=rmsd_fitsq
#ifdef DEBUG
          write (iout,*) "from gauss_fit:",rmsd_gauss
          write (iout,*) "rmsd from dist table",rmsd_dist
#endif
        enddo
      enddo
      call cpu_time(finish)
      write (2,*) "CPU time gaussfit",finish-start," s"
      start=finish
      ii=0
      do icon=1,ncon-1
        do jcon=icon+1,ncon
          rmsd=difconf(icon,jcon)
          ii=ii+1 
          rmsdtab(3,ii)=rmsd
#ifdef DEBUG
          write (iout,*) "from fitsq:",rmsd
#endif
        enddo
      enddo
#ifdef GAUSSFIT
      write (iout,'(5a10)') "Pairs","RMSD_Phi","RMSD_d","RMSD_r",
     & "RMSD_1*"
#else
      write (iout,'(5a10)') "Pairs","RMSD_1","RMSD_d","RMSD_r",
     & "N/A*"
#endif
      ii = 0
      do icon=1,ncon-1
        do jcon=icon+1,ncon
          ii = ii+1
          write (iout,'(2i5,4f10.5)') icon,jcon,rmsdtab(:,ii)
        enddo
      enddo
      call cpu_time(finish)
      write (2,*) "CPU time fitsq",finish-start," s"
      stop
   10 stop "Error in reading pbd file(s)"
      end
c----------------------------------------------------------------------
      subroutine read_pdb(icon,*,*)
      implicit none
      integer i,j,k,ires
      include "DIMENSIONS"
      include "sizesclu.dat"
      include "COMMON.CONTROL"
      include "COMMON.CHAIN"
      include 'COMMON.NAMES'
      include "COMMON.CLUSTER"
      include "COMMON.IOUNITS"
      character*80 karta
      integer icon
      integer rescode

      if (icon.eq.1) then
      nchain=0
      chain_border(1,1)=1
      endif
      ires=0
      do while (ires.lt.maxres)
        read(ipdbin,'(a80)',end=103,err=102) karta
        if (index(karta,'ENDMDL').gt.0) goto 102
c        print *,karta
        if (karta(1:4).eq."ATOM".and.karta(14:15).eq."CA") then
          ires=ires+1
          ireschain(ires)=nchain+1
          itype(ires)=rescode(ires,karta(18:20),0)
          read (karta(31:54),'(3f8.3)',end=202,err=202)
     &          (allcart(j,ires,icon),j=1,3)
        else if (karta(1:3).eq."TER" .and. icon.eq.1) then
          nchain=nchain+1
          chain_border(2,nchain)=ires
          chain_border(1,nchain+1)=ires+1
          chain_length(nchain)=ires-chain_border(1,nchain)+1
        endif
      enddo
  102 continue
c      print *,"102 ",icon
      if (icon.gt.1.and.nres.ne.ires) then
        write (iout,*) "Numbers of residues is ",
     &  " different than in preceding structure(s)",ires,nres
        return1
      endif
      nres=ires
#ifdef DEBUG
      write (iout,*) " Conformation",icon," nchain",nchain
      write (iout,*) "chain_length",chain_length(:nchain)
#endif
c      print *,"chain_border"
c      do i=1,nchain
c        print *,chain_border(1,i),chain_border(2,i)
c      enddo
      if (icon.eq.1) then
      if (nstart.eq.0) nstart=1
      if (nend.eq.0) nend=nres
      nstart_sup=nstart
      nend_sup=nend
c      print *, "Finished broadcast coordinates"
      call chain_symmetry(nchain,nres,itype,chain_border,
     &  chain_length,npermchain,tabpermchain,nchain_group,nequiv,
     &  iequiv,mapchain)
      endif
#ifdef DEBUG
      k=0
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (iout,'(a3,3i4,3f10.5)') restyp(itype(j)),j,
     &     j-chain_border(1,i)+1,i,allcart(:,j,icon)
        enddo
        write (iout,*) 
      enddo
#endif
      return
  103 return2
  202 stop "Error in pdbfile"
      end 
c-------------------------------------------------------------------------
      double precision function gaussdiff(quat)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      include 'COMMON.GAUSSWIDTH'
      double precision quat(0:3)
      double precision przes(3),obrot(3,3),caux(3,maxres)
      double precision rmscalc
      double precision dij,sumgauss,difconf,aux
      integer i,j,k,l,kk1,kk2,ipermmin

      aux=0.0d0
      do i=0,3
        aux=aux+quat(i)*quat(i)
      enddo
      aux=dsqrt(aux)
      quat=quat/aux 
      call quat2rot(quat,obrot)
      call rotate(obrot,c,caux)
#ifdef DEBUG
      do i=1,nres
        write (iout,'(i5,3f10.5,5x,3f10.5)') i,c(:,i),cref(:,i)
      enddo
#endif
      sumgauss=0.0d0
      do i=1,nchain
        do j=1,nchain
#ifdef GAUSSDIST
          aux=0.0d0
          do k=1,chain_length(i)
            kk1=chain_border(1,i)+k-1
            kk2=chain_border(1,j)+k-1
c            print *,i,j,kk1,kk2
            dij=0.0d0
            do l=1,3
              dij=dij+(caux(l,kk1)-cref(l,kk2))**2
            enddo
            aux=aux+dij
            sumgauss=sumgauss+dexp(-0.5d0*dij/sigma2)
          enddo
#ifdef DEBUG
          write (iout,'(f10.5,$)') dsqrt(aux/chain_length(i))
#endif
#else
          dij=0.0d0
          do k=1,chain_length(i)
            kk1=chain_border(1,i)+k-1
            kk2=chain_border(1,j)+k-1
c            print *,i,j,kk1,kk2
            do l=1,3
              dij=dij+(caux(l,kk1)-cref(l,kk2))**2
            enddo
          enddo
          dij=dij/chain_length(i)
          sumgauss=sumgauss+dexp(-0.5d0*dij/sigma2)
#ifdef DEBUG
          write (iout,'(f10.5,$)') dsqrt(dij)
#endif
#endif
        enddo
#ifdef DEBUG
        write (iout,*) 
#endif
      enddo
      gaussdiff=-dlog(sumgauss)
      RETURN
      END
c------------------------------------------------------------------------
      subroutine gaussdiffder(quat,grad)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      double precision quat(0:3),grad(0:3),qi,fplus,fminus
      double precision delta /1.0d-5/
      integer i
      double precision gaussdiff

      do i=0,3
        qi=quat(i)
        quat(i)=qi+delta
        fplus=gaussdiff(quat)
        quat(i)=qi-delta
        fminus=gaussdiff(quat)
        grad(i)=0.5d0*(fplus-fminus)/delta 
        quat(i)=qi
      enddo

      RETURN
      END
c-------------------------------------------------------------------------
      subroutine rotate(obrot,c1,c2)
      implicit none
      include 'DIMENSIONS'
      include 'COMMON.CHAIN'
      integer i,j,k
      double precision obrot(3,3),c1(3,maxres),c2(3,maxres)
      do i=1,nres
        do j=1,3
          c2(j,i)=0.0d0
          do k=1,3
            c2(j,i)=c2(j,i)+obrot(j,k)*c1(k,i)
          enddo
        enddo
      enddo
      return
      end
c-------------------------------------------------------------------------
      subroutine quat2rot(q,obrot)
      implicit none
      double precision q(0:3),obrot(3,3)
      obrot(1,1)=q(0)*q(0)+q(1)*q(1)-q(2)*q(2)-q(3)*q(3)
      obrot(1,2)=2*(q(1)*q(2)-q(0)*q(3))
      obrot(1,3)=2*(q(1)*q(3)+q(0)*q(2))
      obrot(2,1)=2*(q(2)*q(1)+q(0)*q(3))
      obrot(2,2)=q(0)*q(0)-q(1)*q(1)+q(2)*q(2)-q(3)*q(3)
      obrot(2,3)=2*(q(2)*q(3)-q(0)*q(1))
      obrot(3,1)=2*(q(3)*q(1)-q(0)*q(2))
      obrot(3,2)=2*(q(3)*q(2)+q(0)*q(1))
      obrot(3,3)=q(0)*q(0)-q(1)*q(1)-q(2)*q(2)+q(3)*q(3)
      return
      end
c-------------------------------------------------------------------------
      subroutine final_rot(quat,rmsd,lprn)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      include 'COMMON.GAUSSWIDTH'
      double precision quat(0:3)
      double precision przes(3),obrot(3,3)
      double precision rmsd
      double precision dd(maxchain,maxchain),sumgauss,difconf,dij,aux
      integer i,j,k,l,kk1,kk2,ipermmin,nlist,listmin(maxchain),imin,jmin
      logical lprn
      logical chain_present
#ifdef PAIR
      logical pair_present
      integer listpair(2,maxchain)
#endif
      common /permutacja/ listmin
      integer icon,jcon
      common /pair/ icon,jcon
      if (lprn) write (iout,'(a,4f10.5)') 
     &  "Unnormalized quaternion",quat
      aux=0.0d0
      do i=0,3
        aux=aux+quat(i)*quat(i)
      enddo
      aux=dsqrt(aux)
      quat=quat/aux 

      if (lprn) write (iout,'(a,4f10.5)') 
     &  "Final normalized quaternion",quat

      call quat2rot(quat,obrot)

      if (lprn) then
      write (iout,*) "Final rotation matrix"

      do i=1,3
        write (iout,'(3f10.5)') obrot(:,i)
      enddo
      endif

      call rotate(obrot,c,crefjlee)

      do i=1,nchain
        do j=1,nchain
#ifdef DEBUG
          write (2,*) "chains",i,j
#endif
          dij=0.0d0
          do k=1,chain_length(i)
            kk1=chain_border(1,i)+k-1
            kk2=chain_border(1,j)+k-1
#ifdef DEBUG
            write (2,'(2i5,3f10.5,5x,3f10.5)') kk1,kk2,
     &       crefjlee(:,kk1),cref(:,kk2)
#endif
            do l=1,3
              dij=dij+(cref(l,kk1)-crefjlee(l,kk2))**2
            enddo
          enddo
          dd(i,j)=dij/chain_length(i)
          if (lprn) write (iout,'(f10.5,$)') dsqrt(dij/chain_length(i))
        enddo
        if (lprn) write (iout,*) 
      enddo
      nlist=0
      rmsd=0.0d0
#ifdef PAIR
      nlist=0

      do while (nlist.lt.nchain)

      dij=1.0d10

      do i=1,nchain
        do j=1,nchain
          if (pair_present(i,j,nlist,listpair)) cycle
          if (dd(i,j).lt.dij) then
            imin=i
            jmin=j
            dij=dd(i,j) 
          endif 
        enddo
      enddo
      nlist=nlist+1
      listpair(1,nlist)=imin
      listpair(2,nlist)=jmin
      rmsd=rmsd+dij

      enddo

      do i=1,nchain
        do j=1,nchain
          if (listpair(1,j).eq.i) then
            listmin(i)=listpair(2,j)
            goto 11
          endif
        enddo
   11   continue
      enddo
#ifdef DEBUG
      write (iout,*) "nlist",nlist
      write (iout,*) "list_pair1",(listpair(1,i),i=1,nlist)
      write (iout,*) "list_pair2",(listpair(2,i),i=1,nlist)
#endif
#else
      do i=1,nchain
        dij=1.0d10
        do j=1,nchain
          if (chain_present(j,nlist,listmin)) cycle
          if (dd(i,j).lt.dij) then
            jmin=j
            dij=dd(i,j) 
          endif 
        enddo
        nlist=nlist+1
        listmin(nlist)=jmin
c        print *,"i",i," nlist",nlist," listmin",listmin(:nlist)
        rmsd=rmsd+dij
      enddo
#endif
      rmsd=dsqrt(rmsd/nchain)
      if (lprn) then
      write (iout,*) "listmin",listmin(:nlist)
      write (iout,*) "dmin"
      write (iout,*) (dsqrt(dd(i,listmin(i))),i=1,nlist)
      write (iout,*) "rmsd after finding min d",rmsd
      write (iout,'(a)') "REMARK Superposed coordinates"
      write (iout,*) "rmsd",rmsd
      endif
      return
      end
c-------------------------------------------------------------------------
      logical function chain_present(ichain,nlist,listmin)
      include "DIMENSIONS"
      integer ichain,i,nlist,listmin(maxchain)
      if (nlist.eq.0) then
        chain_present=.false.
        return
      endif
      do i=1,nlist
        if (ichain.eq.listmin(i)) then
          chain_present=.true.
          return
        endif
      enddo
      chain_present=.false.
      return
      end
c-------------------------------------------------------------------------
      logical function pair_present(ichain,jchain,nlist,listpair)
      implicit none
      include "DIMENSIONS"
      integer ichain,jchain,i,nlist,listpair(2,maxchain)
      if (nlist.eq.0) then
        pair_present=.false.
        return
      endif
      do i=1,nlist
        if (ichain.eq.listpair(1,i).or.jchain.eq.listpair(2,i)) then
          pair_present=.true.
          return
        endif
      enddo
      pair_present=.false.
      return
      end
c-------------------------------------------------------------------------
      subroutine final_rms
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      double precision quat(0:3)
      double precision przes(3),obrot(3,3)
      double precision rmscalc
      double precision dij,sumgauss,difconf,aux
      integer i,j,k,l,kk1,kk2,ipermmin

      difconf=rmscalc(c(1,1),cref(1,1),przes,obrot,ipermmin)
      write (iout,*) "ipermmin",ipermmin
      write (iout,*) "permutation",tabpermchain(:nchain,ipermmin)
      write (iout,*) "rmsd",difconf
      write (iout,*) "przes",przes
      write (iout,*) "obrot"
      do i=1,3
        write(iout,*) obrot(:,i)
      enddo
      call rotate(obrot,c,crefjlee)
      write (iout,'(a)') "REMARK Superposed coordinates"
      call pdbout(crefjlee)
      do i=1,nchain
        do j=1,nchain
          dij=0.0d0
          do k=1,chain_length(i)
            kk1=chain_border(1,i)+k-1
            kk2=chain_border(1,j)+k-1
c            print *,i,j,kk1,kk2
            do l=1,3
              dij=dij+(crefjlee(l,kk1)-cref(l,kk2))**2
            enddo
          enddo
          write (iout,'(f10.5,$)') dsqrt(dij/chain_length(i))
        enddo
        write (iout,*) 
      enddo
      return
      end
c-------------------------------------------------------------------------
      subroutine orient(c,cm,rot,nres)
      implicit none
      integer nres
      double precision c(3,nres),cm(3),rot(3,3)
      double precision aux,dd,dmax,dij
      integer i,j
      cm(:)=0.0d0
      do i=1,nres 
        cm(:)=cm(:)+c(:,i)
      enddo
      cm(:)=cm(:)/nres
      do i=1,nres
        c(:,i)=c(:,i)-cm(:)
      enddo 
      dmax=0.0d0
      do i=1,nres
        dd=c(1,i)**2+c(2,i)**2+c(3,i)**2
c        print *,"i=",i," dd",dd
        if (dd.gt.dmax) then
          dmax=dd
          do j=1,3
            rot(1,j)=c(j,i)
          enddo
        endif
      enddo
c      print *,"dmax",dmax
      aux=dsqrt(rot(1,1)**2+rot(1,2)**2+rot(1,3)**2)
      do i=1,3
        rot(1,i)=rot(1,i)/aux
      enddo
      dmax=0.0d0
      do i=1,nres
        aux=c(1,i)*rot(1,1)+c(2,i)*rot(1,2)+c(3,i)*rot(1,3)
        dd=c(1,i)**2+c(2,i)**2+c(3,i)**2-aux*aux
c        print *,"i=",i," dd",dd
        if (dd.gt.dmax) then
          do j=1,3
            rot(2,j)=c(j,i)-aux*rot(1,j)
          enddo
          dmax=dd
        endif
      enddo
c      print *,"dmax",dmax
      aux=dsqrt(rot(2,1)**2+rot(2,2)**2+rot(2,3)**2)
      do i=1,3
        rot(2,i)=rot(2,i)/aux
      enddo
      rot(3,1)=rot(1,2)*rot(2,3)-rot(1,3)*rot(2,2)
      rot(3,2)=-rot(1,1)*rot(2,3)+rot(1,3)*rot(2,1)
      rot(3,3)=rot(1,1)*rot(2,2)-rot(1,2)*rot(2,1)
c      do i=1,3
c        do j=1,3
c          print *,i,j,rot(i,1)*rot(j,1)+rot(i,2)*rot(j,2)
c     &     +rot(i,3)*rot(j,3)
c        enddo
c      enddo
c      write (2,*) "Rotation matrix"
c      do i=1,3
c        write (2,*) (rot(i,j),j=1,3)
c      enddo
c
      return
      end
c--------------------------------------------------------------------------
      subroutine initial_quat(q)
      implicit none
      double precision qq(0:3),q(0:3),rmsd,rmsmin
      integer i,j,k,l,nq
      double precision deltaq
      double precision gaussdiff
      nq=2
      deltaq=1.0d0/nq
c      write(2,*) "nq",nq," deltaq",deltaq
      rmsmin=1.0d10
      do i=-nq,nq
        do j=-nq,nq
          do k=-nq,nq
            do l=0,nq
            qq(0)=l*deltaq
            qq(1)=i*deltaq
            qq(2)=j*deltaq
            qq(3)=k*deltaq
#ifdef DEBUG
            write (2,*) i,j,k,l,qq
#endif
            call final_rot(qq,rmsd,.false.)
c            rmsd=gaussdiff(qq)
#ifdef DEBUG
            write (2,*) i,j,k,qq,rmsd,rmsmin
#endif
            if (rmsd.lt.rmsmin) then
              rmsmin=rmsd
              q = qq
            endif
          enddo
          enddo
        enddo
      enddo
c      write (2,*) "q",q," rmsmin",rmsmin
c      call final_rot(q,rmsd,.true.)
      return
      end
c--------------------------------------------------------------------------
      subroutine pdbout(cc)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CHAIN'
      include 'COMMON.NAMES'
      include 'COMMON.IOUNITS'
      character*1 chainid(52) /'A','B','C','D','E','F','G','H','I','J',
     & 'K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
     & 'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p',
     & 'q','r','s','t','u','v','w','x','y','z'/
      double precision cc(3,maxres)
      integer i,j,k,jj,li,listmin(maxchain)
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (igeom,10) j,restyp(itype(j)),chainid(i),
     &      j,(cc(k,j),k=1,3)
        enddo
        write (igeom,'(A)') "TER"
      enddo
      write (igeom,'(A)') "ENDMDL"
  10  FORMAT ('ATOM',I7,'  CA  ',A3,1X,A1,I4,4X,3F8.3,f15.3)
      return
      end
c--------------------------------------------------------------------------
      subroutine calc_rmsd(c1,c2,rmsd)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      include 'COMMON.GAUSSWIDTH'
      double precision rmsd
      double precision dd(maxchain,maxchain),sumgauss,difconf,dij,aux
      integer i,j,k,l,kk1,kk2,ipermmin,nlist,listmin(maxchain),jmin
      logical chain_present
      double precision c1(3,maxres),c2(3,maxres)
#ifdef DEBUG
      do i=1,nres
        write (iout,'(i5,3f10.5,5x,3f10.5)') i,c1(:,i),c2(:,i)
      enddo
#endif
      do i=1,nchain
        do j=1,nchain
          dij=0.0d0
          do k=1,chain_length(i)
            kk1=chain_border(1,i)+k-1
            kk2=chain_border(1,j)+k-1
c            print *,i,j,kk1,kk2
            do l=1,3
              dij=dij+(c1(l,kk1)-c2(l,kk2))**2
            enddo
          enddo
          dd(i,j)=dij/chain_length(i)
#ifdef DEBUG
          write (iout,'(f10.5,$)') dsqrt(dij/chain_length(i))
#endif
        enddo
#ifdef DEBUG
        write (iout,*) 
#endif
      enddo
      nlist=0
      rmsd=0.0d0
      do i=1,nchain
        dij=1.0d10
        do j=1,nchain
          if (chain_present(j,nlist,listmin)) cycle
          if (dd(i,j).lt.dij) then
            jmin=j
            dij=dd(i,j) 
          endif 
        enddo
        nlist=nlist+1
        listmin(nlist)=jmin
c        print *,"i",i," nlist",nlist," listmin",listmin(:nlist)
        rmsd=rmsd+dij
      enddo
      rmsd=dsqrt(rmsd/nchain)
#ifdef DEBUG
      write (iout,*) "listmin",listmin(:nlist)
      write (iout,*) "dmin"
      write (iout,*) (dsqrt(dd(i,listmin(i))),i=1,nlist)
      write (iout,*) "rmsd after finding min d",rmsd
#endif
      return
      end
c------------------------------------------------------------------
      double precision function difconf_gauss(icon,jcon)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.NAMES'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      include 'COMMON.GAUSSWIDTH'
      double precision rmsd_dist,rmsd_fitsq
      common /RMSD/ rmsd_dist,rmsd_fitsq
      integer icon,jcon
      integer i,j,k,iretcode,nfun,imin
      double precision gaussdiff,fg
      double precision rmsd,rmsdmin
      double precision cm(3),cm1(3),cm2(3),rot1(3,3),rot2(3,3)
      double precision q(0:3),grad(0:3),axis(3)
      double precision ffit
      character*3 liczba_i,liczba_j
      do k=nstart,nend
        cref(:,k)=allcart(:,k,icon)
        c(:,k)=allcart(:,k,jcon)
      enddo
#ifdef ORIENT
c shift to origin
      q = 0.0d0
      q(0) = 1.0d0
c      sigma2 = 4.0d0
      call orient(cref,cm1,rot1,nres)
      call rotate(rot1,cref,crefjlee)
      cref(:,:nres)=crefjlee(:,:nres)
      call orient(c,cm2,rot2,nres)
      call rotate(rot2,c,crefjlee)
      c(:,:nres)=crefjlee(:,:nres)
      imin=0
      axis(1)=1.0d0
      axis(2)=1.0d0
      axis(3)=1.0d0
#ifndef INIQUAT
c      call calc_rmsd(c,cref,rmsdmin)
c      rmsdmin=gaussdiff(q)
      call calc_rmsd(c,cref,rmsdmin)
#ifdef DEBUG
      write (iout,*) "0 rmsdmin",rmsdmin
#endif
      do i=1,nres
        c(1,i)=-crefjlee(1,i)
        c(2,i)=-crefjlee(2,i)
        c(3,i)=crefjlee(3,i)
      enddo
      call calc_rmsd(c,cref,rmsd)
c      write (iout,*) "q",q
c      rmsd=gaussdiff(q)
      if (rmsd.lt.rmsdmin) then
        rmsdmin=rmsd
        imin=1
        axis(1)=-1.0d0
        axis(2)=-1.0d0
        axis(3)=1.0d0
      endif
#ifdef DEBUG
      write (iout,*) "1 rmsdmin",rmsdmin," rmsd",rmsd
#endif
      do i=1,nres
        c(1,i)=-crefjlee(1,i)
        c(2,i)=crefjlee(2,i)
        c(3,i)=-crefjlee(3,i)
      enddo
      call calc_rmsd(c,cref,rmsd)
c      write (iout,*) "q",q
c      rmsd=gaussdiff(q)
      if (rmsd.lt.rmsdmin) then
        rmsdmin=rmsd
        imin=2
        axis(1)=-1.0d0
        axis(2)=1.0d0
        axis(3)=-1.0d0
      endif
#ifdef DEBUG
      write (iout,*) "2 rmsdmin",rmsdmin," rmsd",rmsd
#endif
      do i=1,nres
        c(1,i)=crefjlee(1,i)
        c(2,i)=-crefjlee(2,i)
        c(3,i)=-crefjlee(3,i)
      enddo
      call calc_rmsd(c,cref,rmsd)
c      write (iout,*) "q",q
c      rmsd=gaussdiff(q)
      if (rmsd.lt.rmsdmin) then
        rmsdmin=rmsd
        imin=3
        axis(1)=1.0d0
        axis(2)=-1.0d0
        axis(3)=-1.0d0
      endif
#ifdef DEBUG
      write (iout,*) "3 rmsdmin",rmsdmin," rmsd",rmsd
      write (iout,*) "imin",imin," axis",axis," rmsdmin",rmsdmin
#endif
      if (imin.gt.0) then
        do i=1,nres
          do j=1,3
            c(j,i)=axis(j)*crefjlee(j,i)
          enddo
        enddo
      else
        c(:,:nres)=crefjlee(:,:nres)
      imin=0
      endif
c      call calc_rmsd(c,cref,rmsd)
#endif
#else
      cm(:)=0.0d0
      do i=1,nres 
        cm(:)=cm(:)+cref(:,i)
      enddo
      cm(:)=cm(:)/nres
      do i=1,nres
        cref(:,i)=cref(:,i)-cm(:)
      enddo 
      cm(:)=0.0d0
      do i=1,nres 
        cm(:)=cm(:)+c(:,i)
      enddo
      cm(:)=cm(:)/nres
      do i=1,nres
        c(:,i)=c(:,i)-cm(:)
      enddo 
#endif
#ifdef DEBUG
      do i=1,nchain
        do j=chain_border(1,i),chain_border(2,i)
          write (iout,'(a3,3i4,3f10.5,5x,3f10.5)') restyp(itype(j)),j,
     &     j-chain_border(1,i)+1,i,cref(:,j),c(:,j)
        enddo
        write (iout,*) 
      enddo
#endif
c      read (*,*) q
#ifdef INIQUAT
c      sigma2=8.0d0
      call initial_quat(q)
#else
      q = 0.0d0
      q(0)=1.0d0
#endif
#ifdef GAUSSFIT
c      sigma2=8.0d0
c      sigma2=8.0d0
      call minimize(ffit,q(0:3),iretcode,nfun)
#ifdef DEBUG
      write (iout,*) "SUMSL return code",iretcode
      write (iout,*) "target function",ffit," approx rmsd",
     &dsqrt((ffit+dlog(nchain*nchain*chain_length(1)+0.0d0))*nchain*
     &  2*sigma2)
      call final_rot(q,rmsd,.true.)
#endif
      call final_rot(q,rmsd_dist,.false.)
#ifdef PDBOUT
      write (liczba_i,'(bz,i3.3)') icon
      write (liczba_j,'(bz,i3.3)') jcon
      pdbfile=prefix(:len_trim(prefix))//
     &   '_supg_'//liczba_i//'_'//liczba_j//'.pdb'
      open(igeom,file=pdbfile)
      write (igeom,'(a,i5)') "REMARK Structure",icon
      call pdbout(cref)
      write (igeom,'(a,i5)') "REMARK Structure",jcon
      call pdbout(crefjlee)
#endif
#ifdef GAUSSDIST
      difconf_gauss=
     & dsqrt((ffit+dlog(nchain*nchain*chain_length(1)+0.0d0))*2*sigma2)
#else
      difconf_gauss=dsqrt((ffit+dlog(nchain*nchain+0.0d0))*2*sigma2)
#endif
      call rmscalc1(rmsd_fitsq)
c      difconf_gauss=rmsd
#else
      call final_rot(q,rmsd_dist,.false.)
      call rmscalc1(rmsd)
      difconf_gauss=rmsd
#endif
      return
      end
c-------------------------------------------------------------------------
      double precision function difconf(icon,jcon)
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      integer ipermmin
      double precision przes(3),obrot(3,3)
      double precision rmscalc
      integer icon,jcon,k,l
      character*3 liczba_i,liczba_j
c      write (iout,*) "DIFCONF: ICON",icon," JCON",jcon
      do k=1,nres
        do l=1,3
          cref(l,k)=allcart(l,k,icon)
          c(l,k)=allcart(l,k,jcon)
        enddo
c        write (iout,'(i5,3f10.5,5x,3f10.5)') k,cref(:,k),c(:,k)
      enddo
      difconf=rmscalc(c(1,1),cref(1,1),przes,obrot,ipermmin)
#ifdef PDBOUT
      write (liczba_i,'(bz,i3.3)') icon
      write (liczba_j,'(bz,i3.3)') jcon
      pdbfile=prefix(:len_trim(prefix))//
     &   '_supf_'//liczba_i//'_'//liczba_j//'.pdb'
      open(igeom,file=pdbfile)
      write (igeom,'(a,i5)') "REMARK Structure",icon
      call pdbout(cref)
      call rotate(obrot,c,crefjlee)
      do k=1,nres
        crefjlee(:,k)=crefjlee(:,k)+przes(:)
      enddo
      write (igeom,'(a,i5)') "REMARK Structure",jcon
      call pdbout(crefjlee)
#endif
      RETURN
      END
